package eGuideDog::Speaker;

use strict;
use IO::Handle;
use I18N::LangTags::Detect;
use Carp;

use eGuideDog::Basic;

sub new;
sub DESTROY;
sub speak;
sub block_speak;
sub stop;
sub mute;
sub is_playing;
sub is_recording;
sub language;
sub variant;
sub volume;
sub pitch;
sub range;
sub rate;

sub speak_again();
sub speak_next_word();
sub change_style($);
sub speak_item();
sub speak_buffer_item($$);

# constructor
# PARAMETER:
#   tts_engine: 'Festival' or 'eSpeak'(default)
#   config_file: default is './speak.config'
# return eGuideDog::Speaker object reference
sub new {
  my $class = shift; # class name(eGuideDog::Speaker)
  my $tts_engine = shift;
  $tts_engine = 'eSpeak' if (!$tts_engine);
  my $config_file = shift;
  $config_file = './speak.config' if (!$config_file);

  # detect language
  my @languages = I18N::LangTags::Detect::detect();
  my $language = 'en';
  $language = $languages[0] if ($#languages >= 0);

  my $self = {tts_engine => $tts_engine,
	      config_file => $config_file,
	      agent => undef, # speaker object reference
	      record_file => 'speaker.log',
	      recording => 0, # whether it is recording
	      language => $language,
	      mute => 0,
	     };
  bless($self, __PACKAGE__);

  if ($tts_engine eq 'Festival') {
    use eGuideDog::Festival;
    $self->{agent} = eGuideDog::Festival->new();
  } elsif ($tts_engine eq 'eSpeak') {
    use Speech::eSpeak ':all';
    $self->{agent} = Speech::eSpeak->new();
  }

  $self->language($language);

  return $self;
}

sub DESTROY {
  my $self = shift;
}

sub recording {
  my ($self, $recording) = @_;
  $self->{recording} = $recording if (defined $recording);
  return $self->{recording};
}

# speak text
# PARAMETER:
#   $text: speech content
sub speak {
  my $self = shift;
  my $text = shift;

  if (not $self->{mute}) {
    carp('text undefined!') if (!defined $text);

    # record
    if ($self->{recording}) {
      open(FILE, '>>', $self->{record_file});
      binmode(FILE, ':utf8');
      print FILE $text, "\n";
      close(FILE);
    }

    if ($self->{tts_engine} eq 'Festival') {
      $self->{agent}->speak($text);
    } elsif ($self->{tts_engine} eq 'eSpeak') {
      use utf8;
      $self->{agent}->speak($text);
      #    $self->{agent}->synchronize();
    }
  }
}

sub block_speak {
  my $self = shift;
  my $text = shift;

  if (not $self->{mute}) {
    if ($self->{tts_engine} eq 'Festival') {
      $self->{agent}->block_speak($text);
    } elsif ($self->{tts_engine} eq 'eSpeak') {
      $self->{agent}->block_speak($text);
    }
  }
}

sub stop {
  my $self = shift;
  if ($self->{tts_engine} eq 'Festival') {
    $self->{agent}->stop();
  } elsif ($self->{tts_engine} eq 'eSpeak') {
    $self->{agent}->stop();
  }
}

sub mute {
  my ($self, $value) = @_;

  if (defined $value) {
    $self->{mute} = $value;
  } else {
    return $self->{mute};
  }
}

sub is_playing {
  my $self = shift;
  if ($self->{tts_engine} eq 'eSpeak') {
    return $self->{agent}->is_playing();
  }
}

# set or get speech recording status
# PARAMETER:
#   status: 0(default) for not recording, non 0 for recording
# return status if status not specified
sub is_recording {
  my $self = shift;
  my $status = shift;
  if (defined $status) {
    $self->{recording} = $status;
  } else {
    return $self->{recording};
  }
}

sub language {
  my ($self, $language) = @_;
  if ($language) {
    $self->{language} = $language;
    if ($self->{tts_engine} eq 'eSpeak') {
      $self->{agent}->language($language);
    } elsif ($self->{tts_engine} eq 'mute') {
      # do nothing
    } else {
      warn "Don't know how to set language!";
    }
  }
  return $self->{language};
}

# language variant
# 0 for normal
# 1-4 for m1 to m4
# 5-8 for f1 to f4
# 9 for wisper
sub variant {
  my ($self, $variant) = @_;
  my $lang = $self->{agent}->language;
  $lang =~ /(.*)+/;
  $lang = $1 if ($1);
  if ($variant == 0) {
    $lang = $1;
  } elsif ($variant == 1) {
    $lang .= "+m1";
  } elsif ($variant == 2) {
    $lang .= "+m2";
  } elsif ($variant == 3) {
    $lang .= "+m3";
  } elsif ($variant == 4) {
    $lang .= "+m4";
  } elsif ($variant == 5) {
    $lang .= "+f1";
  } elsif ($variant == 6) {
    $lang .= "+f2";
  } elsif ($variant == 7) {
    $lang .= "+f3";
  } elsif ($variant == 8) {
    $lang .= "+f4";
  } elsif ($variant == 9) {
    $lang .= "+wisper";
  }
  warn "change to $lang";
  $self->{agent}->language($lang);
}

sub pitch {
  my ($self, $pitch) = @_;
  return $self->{agent}->pitch($pitch);
}

sub range {
  my ($self, $range) = @_;
  return $self->{agent}->range($range);
}

sub rate {
  my ($self, $rate) = @_;
  return $self->{agent}->rate($rate);
}

sub volume {
  my ($self, $volume) = @_;
  return $self->{agent}->volume($volume);
}

# speak the last sentence again
sub speak_again() {
#  speak($sentences[$sentence_index]);
}

# speak the next word of current
sub speak_next_word() {
}


# change the voice style, pitch, strong for example
sub change_style($) {
  my $style = shift;
#  print FESTIVAL "(Style '$style)";
}

# speak current item
sub speak_item() {
  speak($eGuideDog::index{sound_buffer}->[$eGuideDog::index{current_item}]);
}

sub speak_buffer_item($$) {
  my $buffer = shift;
  my $line = shift;
  speak($buffer->[$line]);
}

1;
